// -*- C++ -*-
/*!
 * @file  UVCCameraViewer.cpp
 * @brief UVCCamera Component Viewer
 * @date $Date$
 *
 * @author メディアドライブ
 * rtc-dev@mediadrive.jp
 *
 * BSD 3-clause "New" or "Revised" License
 *
 * $Id$
 */

#include "UVCCameraViewer.h"
#include "ImageDataConvert.h"

// Module specification
// <rtc-template block="module_spec">
static const char* uvccameraviewer_spec[] =
  {
    "implementation_id", "UVCCameraViewer",
    "type_name",         "UVCCameraViewer",
    "description",       "UVCCamera Component Viewer",
    "version",           "1.0.0",
    "vendor",            "MEDIA DRIVE",
    "category",          "ImageProcessing",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.image_height", "480",
    "conf.default.image_width", "640",

    // Widget
    "conf.__widget__.image_height", "text",
    "conf.__widget__.image_width", "text",
    // Constraints

    "conf.__type__.image_height", "int",
    "conf.__type__.image_width", "int",

    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
UVCCameraViewer::UVCCameraViewer(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_inCamIn("inCam", m_inCam)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
UVCCameraViewer::~UVCCameraViewer()
{
}


/*!
 * 各変数の初期化を行います
 */
RTC::ReturnCode_t UVCCameraViewer::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("inCam", m_inCamIn);

  // Set OutPort buffer

  // Set service provider to Ports

  // Set service consumers to Ports

  // Set CORBA Service Ports

  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("image_height", m_image_height, "480");
  bindParameter("image_width", m_image_width, "640");
  // </rtc-template>

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t UVCCameraViewer::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t UVCCameraViewer::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t UVCCameraViewer::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
 * 準備を行います
 */

RTC::ReturnCode_t UVCCameraViewer::onActivated(RTC::UniqueId ec_id)
{
    //m_in_height = 0;
    //m_in_width  = 0;


  //画像表示用ウィンドウの作成
    cvNamedWindow("Gorone", CV_WINDOW_AUTOSIZE);
    return RTC::RTC_OK;
}

/*!
 * メモリの開放を行います
 */

RTC::ReturnCode_t UVCCameraViewer::onDeactivated(RTC::UniqueId ec_id)
{
  cvDestroyWindow("Gorone");
  return RTC::RTC_OK;
}

/*!
 * Inportに送られてきたデータを表示します
 */

RTC::ReturnCode_t UVCCameraViewer::onExecute(RTC::UniqueId ec_id)
{
  static coil::TimeValue tm_pre;
  //static int count = 0;
  if (m_inCamIn.isNew())
  {
  	m_inCamIn.read();
  	std::string str = "Gorone";
  	if(ImageProgress(m_inCam, str) < 0)
  		return RTC::RTC_OK;
  }
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t UVCCameraViewer::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t UVCCameraViewer::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t UVCCameraViewer::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t UVCCameraViewer::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t UVCCameraViewer::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

int UVCCameraViewer::ImageProgress(RTC::CameraImage& m_in, std::string strGoron)
{
    if (!(m_in.pixels.length() > 0))
    {
    	std::cout << "[ImageProgress] Size is 0" << std::endl;
        return -1;
    }

    cv::Mat orig_img = GetMatImage(m_in);
    //cv::Mat small_img = cv::Mat::zeros(cv::Size(m_image_width, m_image_height), CV_8UC3);
    cv::Mat small_img = cv::Mat::zeros(cv::Size(orig_img.cols, orig_img.rows), CV_8UC3);

#if 0
    std::cout << "orig_img"<< std::endl;
     std::cout << "  dims: " << orig_img.dims << ", depth(byte/channel):" << orig_img.elemSize1() \
   	    << ", channels: " << orig_img.channels() << std::endl;
     std::cout << "size width: " << orig_img.cols << ", height: " << orig_img.rows << std::endl;
     std::cout << "isContinuous:" << (orig_img.isContinuous()?"true":"false") << std::endl;

     std::cout << "small_img"<< std::endl;
     std::cout << "  dims: " << small_img.dims << ", depth(byte/channel):" << small_img.elemSize1() \
   	    << ", channels: " << small_img.channels() << std::endl;
     std::cout << "size width: " << small_img.cols << ", height: " << small_img.rows << std::endl;
     std::cout << "isContinuous:" << (small_img.isContinuous()?"true":"false") << std::endl;
#endif
   //変換
    cv::resize(orig_img, small_img, small_img.size(), 0, 0, cv::INTER_LINEAR);
    //画像表示
//	#if (!defined WIN32) || (!defined WIN64)
 //   cvStartWindowThread();
//	#endif
    cv::imshow(strGoron.c_str() , small_img);
    cv::waitKey(2);
return 0;
}

extern "C"
{

  void UVCCameraViewerInit(RTC::Manager* manager)
  {
    coil::Properties profile(uvccameraviewer_spec);
    manager->registerFactory(profile,
                             RTC::Create<UVCCameraViewer>,
                             RTC::Delete<UVCCameraViewer>);
  }

};


